# -*- coding: utf-8 -*-
import law
import luigi
import logging
import os
import re

from functools import partial

from analysis.tasks.Warmup import Warmup

from analysis.framework import Task, SpaceSeparatedListParameter
from analysis.mixins import TarballExtractionMixin
from analysis.util import working_directory

logger = logging.getLogger(__name__)
working_directory = partial(working_directory, logger=logger)


class CopyWarmup(Task, TarballExtractionMixin, law.LocalWorkflow):
    force = luigi.BoolParameter(default=False)

    warm_dir = luigi.Parameter(default='warmups')

    default_channels = ["LO", "R", "V", "RRa", "RRb", "RV", "VV"]
    channels = SpaceSeparatedListParameter(
        default=default_channels,
        cls=luigi.Parameter,
        choices=("LO", "R", "V", "RR", "RRa", "RRb", "RV", "VV")
    )
    jobnameext = luigi.Parameter()
    cutoff = luigi.Parameter()

    def create_branch_map(self):
        """Branch data dicts. Each branch should correspond to exactly one output."""
        branchmap = {}
        for i, channel in enumerate(self.channels):
            branchmap[i] = {
                'index': i,
                'channel': channel
            }
        return branchmap

    def workflow_requires(self):
        if self.force:
            return {"warmup": Warmup.req(self, acceptance=0)}
        else:
            return {"warmup": Warmup.req(self)}

    def requires(self):
        return {'warmup': Warmup(branch=self.branch_data['index'])}

    def output(self):
        out_dir = os.getenv("ANALYSIS_PATH") or os.getcwd()
        return law.LocalFileTarget(
            '{}/{}/{}/{}/{}.{}-{}.{}.{}'.format(
                out_dir,
                self.warm_dir, 'warmup', self.name,
                self.process, self.branch_data['channel'],
                self.jobnameext, self.cutoff, self.branch_data['channel'],
            )
        )

    def run(self):
        # get input and output targets
        input_ = Warmup().output()['collection'].targets[self.branch]
        output = self.output()

        # extract input tarball to output parent directory
        self.extract_tarball(
            input_,
            output.parent,
            formatter="tar",
            logger=logger,
            # check that expected file was extracted
            check_targets=[
                output,
            ],
        )
